# Overview

This is a subordinate charm to deploy the telegraf metrics agent to collect
metrics from related services. The metrics may be scraped by or pushed
to supported systems such as Prometheus, Graphite or InfluxDB.

For details about telegraf see: https://github.com/influxdata/telegraf 

# Usage

Deploy telegraf alongside your service, and also a time series storage (in this case, influxdb)

    juju deploy telegraf 
    juju deploy influxdb 
    juju deploy some-service

Add the relations: 

    juju add-relation telegraf:juju-info some-service:juju-info 
    juju add-relation telegraf:influxdb-api influxdb:api


# Configuration

## Prometheus Output

By default, Prometheus output is presented available for scraping.

## InfluxDB Output

When related to an InfluxDB service, metrics will be pushed to a database
named ``telegraf``.

## Apache Input

For the apache input plugin, the charm provides the apache relation which
uses ``apache-website`` interface. Current apache charm disables
``mod_status`` and in order to telegraf apache input to work ``status``
should be removed from the list of ``disable_modules`` in the apache charm
config.

## Postgresql & MySQL input

Due to [relation scope issues](https://bugs.launchpad.net/postgresql-charm/+bug/1560262),
two relations need to be established between telegraf and the postgresql
service. first a plain ``juju-info`` relation to get standard system
metrics and a ``db-admin`` relation for PostgreSQL database metrics. eg.:

    ```
    juju add-relation postgresql:juju-info telegraf
    juju add-relation postgresql:db-admin telegraf
    ```

Similarly, for MySQL:

    ```
    juju add-relation mysql:juju-info telegraf
    juju add-relation mysql:db-admin telegraf
    ```

## Advanced

### Input

A basic set of input plugins are setup by default, which can be overriden
with the ``inputs_config`` configuration option. Both of these accept raw
telegraf configuration.

To configure any of the (default or via relation) plugins, the
``extra_options`` charm config can be used. It's yaml, that needs to
be encoded as a string in the config. For example:

    ```yaml
    telegraf:
    # The trailing | means the following indented section is all one string.
    extra_options: |
        inputs:
            cpu:
                percpu: false
                fielddrop: ["time_*"]
            disk:
                mount_points: ["/"]
                ignore_fs: ["tmpfs", "devtmpfs"]
            elasticsearch:
                local: false
                cluster_health: true
            postgresql:
                databases: ["foo", "bar"]
                tagpass: 
                    db: ["template", "postgres"]
        outputs:
            influxdb:
                precision: ms
    ```

This extra options will only be applied to plugins configured via
relations and the following list (defined in ``templates/base_inputs.conf``):
    - inputs.cpu
    - inputs.disk
    - inputs.diskio
    - inputs.mem
    - inputs.net
    - inputs.netstat
    - inputs.swap
    - inputs.system


### Output 

The only output plugin supported via relation is influxdb, any other output
plugin needs to be configured manually (via juju set).

To use a different metrics storage such as Graphite, the raw telegraf
plugin configuration needs to be set as a string in ``outputs_config``
configuration. For example:

    ```yaml
    telegraf:
      # The trailing | indicates a multiline string in yaml.
      outputs_config: |
        [[outputs.graphite]]
        servers = ["10.0.3.231:2003"]
        prefix = "juju_local.devel.telegraf"
        timeout = 10
    ```

This will make telegraf agents to send the metrics to the graphite instance.

# Contact Information

- Prometheus Charming Project: https://launchpad.net/prometheus-charms
- Charm bugs: https://bugs.launchpad.net/telegraf-charm
- Upstream https://github.com/influxdata/telegraf
- Upstream bug tracker https://github.com/influxdata/telegraf/issues
