#!/usr/bin/python3
#
# Copyright 2015-2016 Canonical Ltd.
#
# This file is part of the Telegraf Charm for Juju.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3, as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import subprocess
import unittest

import amulet
import requests


class TestPostgreSQL(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        # amulet only detects the charm name correctly if run from
        # the root directory. Work around this.
        os.environ['JUJU_TEST_CHARM'] = 'telegraf'
        charm_dir = os.path.join(os.path.dirname(__file__), os.pardir)

        cls.d = amulet.Deployment(series=os.environ.get('SERIES', 'xenial'))

        cls.d.add('postgresql', 'cs:postgresql', units=2)
        cls.d.add('telegraf', charm_dir)
        cls.d.expose('telegraf')
        cls.d.relate('postgresql:juju-info', 'telegraf:juju-info')
        cls.d.relate('postgresql:db', 'telegraf:postgresql')

        cls.d.setup(timeout=900)
        cls.d.sentry.wait()

        subprocess.check_call(['juju', 'wait'], universal_newlines=True)

    def test_service(self):
        for sentry in self.d.sentry['telegraf']:
            with self.subTest(unit=sentry.info['unit_name']):
                # test we can access over http
                addr = sentry.info['public-address']
                url = 'http://{}:9103/metrics'.format(addr)
                page = requests.get(url)
                self.assertEqual(page.status_code, 200)
                # Confirm PostgreSQL specific metrics exist.
                self.assertIn('postgresql_blks_hit', page.text)
                # Confirm general metrics exist.
                self.assertIn('cpu_usage_idle', page.text)


if __name__ == '__main__':
    unittest.main()
